#include <SPI.h>

/*
*
*  Weather Clock
*
*/

#include <Flash.h>;
#include <Ethernet.h>;
#include <Servo.h>;
#include <Streaming.h>;

#define endl "\n";

//
// Ethernet variables
//

byte arduinoMac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xEA };
byte arduinoIp[] = { 10, 0, 0, 4 };
byte serverIp[] = { 10, 0, 0, 2 };

Server server(80);
Client fetchClient(serverIp, 80);

//
// Weather conditions
//

int8_t temperature = 99;
int8_t windchill = 99;
String currentCondition = String(30);
String currentConditionLower = String(30);
int8_t currentConditionImage = 0;
String wind = String(15);
int8_t humidity = 101;
String forecastCondition = String(30);
String forecastConditionLower = String(30);
int8_t forecastConditionImage = 0;
int8_t forecastHigh = 99;
int8_t forecastLow = 99;
int8_t forecastPOP = 0;
int time = 0;
int updateAfterTime = 700;
int updateBeforeTime = 2200;
boolean useWindchill = 0;

Servo servoCondition;
Servo servoTemperature;

uint8_t servoConditionValue = 0;
uint8_t servoTemperatureValue = 0;
uint8_t oldServoConditionValue = 90;
uint8_t oldServoTemperatureValue = 90;

byte maxLength = 50;
String inString = String(maxLength);

int updatecount = 890;
long previousMillis = 0;



void setup() {
  
//   Start serial for debugging
  Serial.begin(57600);
  
  Serial.println("Up");

  servoTemperature.attach(5);
  delay(1000);
  servoCondition.attach(6);
  
  servoCondition.write(90);
  servoTemperature.write(90);
    
  delay(1000);
  
  servoTemperature.detach();
  servoCondition.detach();
  
  // Delay until ethernet initalizes
  delay(5000);
  
  // Start ethernet
  Ethernet.begin(arduinoMac, arduinoIp);
  server.begin();
  

  

  
}

void loop() {
  
  if (millis() - previousMillis > 1000) {
    previousMillis = millis();
    // One second has elapsed, tick the clock.
    updatecount++;
  }
  
  if (updatecount >= 900) {
    fetchWeather();
  }
  
  Client client = server.available();
  if (client) {
    boolean current_line_is_blank = true;
    while (client.connected()) {
      if (client.available()) {
        char c = client.read();
        if (inString.length() < maxLength) {
          inString.concat(c);
        }
        if (c == '\n' && current_line_is_blank) {
          if (inString.indexOf("?") != -1) {
            
            int Pos_a = inString.indexOf("a");
            int Pos_b = inString.indexOf("b");
            int Pos_c = inString.indexOf("c");
            int End = inString.indexOf("H");
            updateAfterTime = getInt(inString.substring((Pos_a+2), (Pos_b-1)));
            updateBeforeTime = getInt(inString.substring((Pos_b+2), (Pos_c-1)));
            useWindchill = getInt(inString.substring((Pos_c+2), (End-1)));
            
            updateClock();
           
          }
          client << F("HTTP/1.1 200 OK") << endl;
          client << F("Content-Type: text/html") << endl;
          client << endl;
          client << F("<html><head><title>Winnipeg Weather</title></head><body>") << endl;
          client << F("<h1>Winnipeg Weather</h1>") << endl;
          client << F("<h2>Current Conditions</h2>") << endl;
          client << F("<table><tr><td width=130px><img src='http://www.seancarney.ca/backend/weather-clock/icons/0") << currentConditionImage << F(".png'></td><td>") << endl;
          client << F("<h3>"); 
          client.print(currentCondition);
          client << F("</h3>") << endl;
          client << F("Temperature: ") << temperature;
          if ((windchill != -1) && (windchill != 99) && (windchill != 0)) {
            client << F("&deg; (") << windchill << F("&deg; with wind)<br>") << endl;
          } else {
            client << "&deg;<br>" << endl; }
          client << F("Wind: ");
          client.print(wind);
          client << F("<br>") << endl;
          client << F("Humidity: ") << humidity << F("%</td></tr></table>") << endl;
          client << F("<h2>Forecast</h2>") << endl;
          client << F("<table><tr><td width=130px><img src='http://www.seancarney.ca/backend/weather-clock/icons/0") << forecastConditionImage << F(".png'></td><td>") << endl;
          client << F("<h3>"); 
          client.print(forecastCondition);
          client << F("</h3>") << endl;
          client << F("Daytime high: ") << forecastHigh << F("&deg;<br>") << endl;
          client << F("Nighttime low: ") << forecastLow << F("&deg;<br>") << endl;
          client << F("Probability of precipitation: ") << forecastPOP << F("%</td></tr></table>") << endl;
          client << F("<h2>Settings</h2>") << endl;
          client << F("<form method=get>Only update the clock between <input type=text size=4 value=\"") << updateAfterTime << F("\" name=a> and <input type=text size=4 value=\"") << updateBeforeTime << F("\" name=b><br>");
          client << F("Show windchill adjusted temperature <input type=radio name=\"c\" value=1 ");
          if (useWindchill == 1)
            client << F("checked ");
          client << F("> Yes <input type=radio name=\"c\" value=0 ");
          if (useWindchill == 0)
            client << F("checked ");
          client << F("> No<br><br><input type=submit value=Update></form>");
          client << F("<p><i>Last updated: ") << time << F("</i></p>") << endl;
          client << F("</body></html>") << endl; 
          break;
        }
        if (c == '\n') {
          current_line_is_blank = true;
        } else if (c != '\r') {
          current_line_is_blank = false;
        }
        
      }
      
    }
    
    delay(50);  // this may be why wireless isn't working - need to increase the timeout
    inString = "";
    client.flush();
    client.stop();
  
  }
  
}

void fetchWeather() {
  
  if (fetchClient.connect()) {
    Serial << F("Reconnected");
    fetchClient.println("GET http://www.seancarney.ca/backend/weather-clock/xml/canada-manitoba-winnipeg.php HTTP/1.0");
    fetchClient.println();
    delay(2000);
    updatecount = 0;
    
  windchill = 99;
  forecastHigh = 99;
  forecastLow = 99;
  forecastPOP = 0;
  
  while (fetchClient.available()) {
    char c = fetchClient.read();
        if (inString.length() < maxLength) {
          inString.concat(c);
        }
        if (c == '\n') {
          Serial.println(inString);
          if (inString.startsWith("<temperature>")) {
            temperature = getInt(inString.substring(13, inString.length()-15));
            Serial.println(temperature);
          }
          if (inString.startsWith("<windchill>"))
            windchill = getInt(inString.substring(11, inString.length()-13));
          if (inString.startsWith("<currcond>"))
            currentCondition = inString.substring(10, inString.length()-12);
          if (inString.startsWith("<wind>"))
            wind = inString.substring(6, inString.length()-8);
          if (inString.startsWith("<humidity>"))
            humidity = getInt(inString.substring(10, inString.length()-12));
          if (inString.startsWith("<forecond>"))
            forecastCondition = inString.substring(10, inString.length()-12);
          if (inString.startsWith("<temphigh>"))
            forecastHigh = getInt(inString.substring(10, inString.length()-12));
          if (inString.startsWith("<templow>"))
            forecastLow = getInt(inString.substring(9, inString.length()-11));
          if (inString.startsWith("<pop>"))
            forecastPOP = getInt(inString.substring(5, inString.length()-7));
          if (inString.startsWith("<time>"))
            time = getInt(inString.substring(6, inString.length()-8));
          
          inString = "";
        }
  }
  
  fetchClient.stop();
  
  updateClock();

    
  } else {
    Serial << F("Reconnect failed");
    fetchClient.flush();
    fetchClient.stop();
    updatecount = 890;
  }
  
}


void updateClock() {
  
  /*Servo positions
  
  18 = snow
  38 = sleet
  58 = cloudy
  81  = partly cloudy
  103 = sunny
  126 = scattered showers
  146 = showers
  166 = thundershowers
  
  12 = -40
  170 = 40
  
  0 = +90
  
  (temp*2)+90
  
  */
  
  servoConditionValue = 90;
  currentConditionImage = 0;
  forecastConditionImage = 0;
    
  currentConditionLower = currentCondition.toLowerCase();
//  currentConditionLower.toLowerCase();
  
  forecastConditionLower = forecastCondition.toLowerCase();
//  forecastConditionLower.toLowerCase();
  
  Serial.println(currentConditionLower);
    
  if (currentConditionLower.indexOf("cloudy") != -1) {
    servoConditionValue = 58;
    currentConditionImage = 3;
  }
  if ((currentConditionLower.indexOf("partly cloudy") != -1) || currentConditionLower.indexOf("mostly cloudy") != -1) {
    servoConditionValue = 81;
    currentConditionImage = 2;
  }
  if ((currentConditionLower.indexOf("sunny") != -1) || currentConditionLower.indexOf("clear") != -1) {
    servoConditionValue = 103;
    currentConditionImage = 1;
  }
  if ((currentConditionLower.indexOf("snow") != -1)) {
    servoConditionValue = 18;
    currentConditionImage = 8;
  }
  if ((currentConditionLower.indexOf("crystals") != -1) || (currentConditionLower.indexOf("fog") != -1)) {
    servoConditionValue = 81;
    currentConditionImage = 2;
  }
  if ((currentConditionLower.indexOf("rain") != -1) || (currentConditionLower.indexOf("drizzle") != -1) || (currentConditionLower.indexOf("precipitation") != -1)) {
    servoConditionValue = 146;
    currentConditionImage = 5;
  }
  if ((currentConditionLower.indexOf("rain and snow") != -1) || (currentConditionLower.indexOf("freezing rain") != -1) || (currentConditionLower.indexOf("freezing drizzle") != -1)) {
    servoConditionValue = 38;
    currentConditionImage = 7;
  }
  if (currentConditionLower.indexOf("distant precipitation") != -1) {
    servoConditionValue = 126;
    currentConditionImage = 4;
  }
  
 
  if (forecastConditionLower.indexOf("cloud") != -1)
    forecastConditionImage = 3;
  if ((forecastConditionLower.indexOf("sunny") != -1) || (forecastConditionLower.indexOf("clear") != -1))
    forecastConditionImage = 1;
  if ((forecastConditionLower.indexOf("sunny") != -1) && (forecastConditionLower.indexOf("cloud") != -1))
    forecastConditionImage = 2;
  if ((forecastConditionLower.indexOf("crystals") != -1) || (forecastConditionLower.indexOf("fog") != -1))
    forecastConditionImage = 2;
  if ((forecastConditionLower.indexOf("showers") != -1) || (forecastConditionLower.indexOf("rain") != -1) || (forecastConditionLower.indexOf("drizzle") != -1) || (forecastConditionLower.indexOf("precipitation") != -1))
    forecastConditionImage = 5;
  if ((forecastConditionLower.indexOf("snow") != -1) || (forecastConditionLower.indexOf("flurries") != -1))
    forecastConditionImage = 8;
  if ((forecastConditionLower.indexOf("rain and snow") != -1) || (forecastConditionLower.indexOf("freezing rain") != -1) || (forecastConditionLower.indexOf("freezing drizzle") != -1))
    forecastConditionImage = 7;
  if (forecastConditionLower.indexOf("distant precipitation") != -1)
    forecastConditionImage = 4;
  
  if ((time > updateAfterTime)&&(time < updateBeforeTime)) {
    
    if ((useWindchill == 0)||(windchill == 99)||(windchill == -1))
      servoTemperatureValue = ((temperature * 2) + 90);
    else
      servoTemperatureValue = ((windchill * 2) + 90);
    
    // Sanity check
    
    if (servoTemperatureValue < 12)
      servoTemperatureValue = 12;
    if (servoTemperatureValue > 170)
      servoTemperatureValue = 170;
      
    if (servoConditionValue < 15)
      servoConditionValue = 15;
    if (servoConditionValue > 175)
      servoConditionValue = 175;
      
    if (servoConditionValue != oldServoConditionValue) {
      servoCondition.attach(6);
      if (servoConditionValue > oldServoConditionValue)
        for (int i = oldServoConditionValue; i != servoConditionValue; i++)
        {
          servoCondition.write(i);
          delay(100);
        }
      else
        for (int i = oldServoConditionValue; i != servoConditionValue; i--)
        {
          servoCondition.write(i);
          delay(100);
        }
      delay(1000);
      servoCondition.detach();
      oldServoConditionValue = servoConditionValue;
    }
    
    if (servoTemperatureValue != oldServoTemperatureValue) {
      servoTemperature.attach(5);
      if (servoTemperatureValue > oldServoTemperatureValue)
        for (int i = oldServoTemperatureValue; i != servoTemperatureValue; i++)
        {
          servoTemperature.write(i);
          delay(100);
        }
      else
        for (int i = oldServoTemperatureValue; i != servoTemperatureValue; i--)
        {
          servoTemperature.write(i);
          delay(100);
        }
      servoTemperature.detach();
      oldServoTemperatureValue = servoTemperatureValue;
    }
    
    
  }
  
  
}


int getInt(String text)
{
  char tempqa[20];
  text.toCharArray(tempqa, 19);
  int x = atoi(tempqa);
  if (x == 0 && text != "0")
  {
    x = -1;
  }
  return x;
}  
